--[[
An ActionButton that has a defined behavior of "do nothing".  Normally, this
shows up as a typical cancel button (x), but can show up as a menu selection
with the word "Cancel" associated with it.  Other looks are also possible.

There is one ActionButtonNone per UnitGroupHeader.
Each ActionButtonNone instance is associated with a single UnitGroupHeader.
--]]
uab.classes.ActionButtonNone = uab.mixins.aceOO.Class()

function uab.classes.ActionButtonNone.ShowTooltip(frame)
    if uab.db.profile.tooltips then
        GameTooltip:SetOwner(frame, "ANCHOR_RIGHT")
        GameTooltip:SetBackdropColor(0.1, 0.1, 0.1, 0.8)
        GameTooltip:SetText("Cancel unit action", nil, nil, nil, nil, true)
    end
end

function uab.classes.ActionButtonNone.prototype:init(header)
	uab.classes.ActionButtonNone.super.prototype.init(self)
	
	self.header = header
    self.posstates = {}
    self.showstates = {}

    self.framename = "uab_"..self.header.name.."_ActionBtnNone"
    self.frame = getglobal(self.framename)

    if not self.frame then
    	--[[
    	Create frame
    	--]]
        self.frame = CreateFrame("Button", self.framename, nil, "UAB_StandardCancelButtonTemplate")
    end
    
    self.frame:SetFrameLevel(1)
    self.frame:SetFrameStrata("FULLSCREEN_DIALOG")
    self.frame:Hide()
    
    uabActionButton:SetFrameAttribute(self.frame, "newstate", "*:0")
    
    self.frame.uabobj = self

    uabActionButton:SetBoolFrameAttribute(self.frame, "useparent-unit", true)
    uabActionButton:SetBoolFrameAttribute(self.frame, "useparent-unitsuffix", true)
    uabActionButton:SetFrameAttribute(self.frame, "state", "0")

    self:Enable()
	self:SetParent()
    
    self.frame:SetPoint("CENTER", 0, 0)
    self.frame:SetScript("OnEnter", function(self, motion) uab.classes.ActionButtonNone.ShowTooltip(self) end)
    self.frame:SetScript("OnShow", function(self) self.uabobj:OnShow() end)
    self.frame:SetScript("OnLeave", function(self, motion) GameTooltip:Hide() end)

    --[[
    Start out with the button always hidden.  When some frame registers for the
    unit group of the activator for this button, the display properties of this
    button will get adjusted then.  We just need to start by indicating that the
    button is always hidden.
    --]]
    self:PrepareForLayouts()
    self:LayoutsDone()
end

function uab.classes.ActionButtonNone.prototype:Dump(dumper)
    dumper:Dump("Cancel: "..self.framename)
    dumper:IncIndent()
    if self.header then
        dumper:Dump("SecureHeader: "..self.header.frame:GetName())
    else
        dumper:Dump("SecureHeader: none")
    end
    dumper:DumpFrameAttribute(self.frame, "newstate")
    dumper:DumpFrameAttribute(self.frame, "ofspoint")
    dumper:DumpFrameAttribute(self.frame, "ofsx")
    dumper:DumpFrameAttribute(self.frame, "ofsy")
    dumper:DumpFrameAttribute(self.frame, "showstates")
    dumper:DumpFrameAttribute(self.frame, "hidestates")
    dumper:DecIndent()
end

function uab.classes.ActionButtonNone.prototype:PrepareForLayouts()
    for idx, _ in pairs(self.showstates) do
        self.showstates[idx] = nil
    end

    for x, _ in pairs(self.posstates) do
        self.posstates[x].pt = nil
        self.posstates[x].x = nil
        self.posstates[x].y = nil
        self.posstates[x] = nil
    end
end

function uab.classes.ActionButtonNone.prototype:AddShowStates(states)
    self.showstates[states] = true
end

function uab.classes.ActionButtonNone.prototype:SetPointForStates(states, ofspoint, ofsx, ofsy)
    self.posstates[states] = { pt = ofspoint, x = ofsx, y = ofsy }
end

function uab.classes.ActionButtonNone.prototype:LayoutsDone()
    local ptattr, xattr, yattr
    
    for posstates, pos in pairs(self.posstates) do
        if not ptattr then
            ptattr = ""
            xattr = ""
            yattr = ""
        else
            ptattr = ptattr .. ";"
            xattr = xattr .. ";"
            yattr = yattr .. ";"
        end

        ptattr = ptattr .. posstates .. ":" .. pos.pt
        xattr = xattr .. posstates .. ":" .. pos.x
        yattr = yattr .. posstates .. ":" .. pos.y
    end

    uabActionButton:SetFrameAttribute(self.frame, "ofspoint", ptattr)
    uabActionButton:SetFrameAttribute(self.frame, "ofsx", xattr)
    uabActionButton:SetFrameAttribute(self.frame, "ofsy", yattr)

    local showstates

    for states, _ in pairs(self.showstates) do
        if not showstates then
            showstates = ""
        else
            showstates = showstates .. ","
        end

        showstates = showstates .. states
    end

    if not showstates then
        uabActionButton:SetFrameAttribute(self.frame, "showstates", nil)
        uabActionButton:SetFrameAttribute(self.frame, "hidestates", "*")
    else
        uabActionButton:SetFrameAttribute(self.frame, "showstates", showstates)
        uabActionButton:SetFrameAttribute(self.frame, "hidestates", nil)
    end
end

function uab.classes.ActionButtonNone.prototype:SetParent()
    if self.header.frame then
        self.frame:SetParent(self.header.frame)
        self.frame:SetFrameLevel(1)
        self.frame:SetFrameStrata("FULLSCREEN_DIALOG")
        uabActionButton:SetFrameFrameAttribute(self.header.frame, "addchild", self.frame)
    end
end

function uab.classes.ActionButtonNone.prototype:Show()
	-- Nothing to do here, since this is the cancel button, which does not
	-- change visual appearance.
end

function uab.classes.ActionButtonNone.prototype:Hide()
	self.frame:Hide()
end


function uab.classes.ActionButtonNone.prototype:Enable()
end

function uab.classes.ActionButtonNone.prototype:Disable()
end

function uab.classes.ActionButtonNone.prototype:Frame()
	return self.frame
end

function uab.classes.ActionButtonNone.prototype:OnShow()
    self.frame:SetFrameStrata("FULLSCREEN_DIALOG")
end

