--[[
Main class for the add-on.  Handles all run-time functionality.
--]]

uabMain = uab:NewModule("Main", "AceEvent-2.0")
uabMain.core = uab

uabDumper = {}
uabDumper.dumplines = {}
uabDumper.indent = 0

function uabDumper:Clear()
    for idx, text in pairs(self.dumplines) do
        self.dumplines[idx] = nil
    end
end

function uabDumper:IncIndent()
    self.indent = self.indent + 1
end

function uabDumper:DecIndent()
    self.indent = self.indent - 1
end

function uabDumper:DumpFrameAttribute(frame, attr)
    local hdr = "Attribute ("..frame:GetName()..") - "..attr
    local val = frame:GetAttribute(attr)
    if not val then
        self:Dump(hdr.." : nil")
    elseif type(val) == "string" then
        self:Dump(hdr.." : "..val)
    elseif type(val) == "boolean" then
        self:Dump(hdr.." : true")
    elseif type(val) == "table" and val.GetName then
        self:Dump(hdr.." : "..val:GetName())
    else
        self:Dump(hdr.." : raw value")
    end
end

function uabDumper:Dump(text)
    local dumptext = ""
    for idx = 1, self.indent, 1 do
        dumptext = dumptext .. "    "
    end
    dumptext = dumptext .. text
    
    table.insert(self.dumplines, dumptext)
end

function uabDumper:Write()
    for idx, text in pairs(UABDebug) do
        UABDebug[idx] = nil
    end
    
    for idx, text in pairs(self.dumplines) do
        table.insert(UABDebug, self.dumplines[idx])
    end
end

function uabMain:OnInitialize()
	uabMain.super.OnInitialize(self)

    self.secureheader = uab.classes.SecureHeader:GetHeader()
    
    self.name = "uab.main"	
	self.activators = {}
	self.combatRegistrations = {}
	self.combatUnregistrations = {}

    self:RegisterEvent("PLAYER_DEAD")
    self:RegisterEvent("PLAYER_REGEN_ENABLED")
end

function uabMain:Dump()
    uabDumper:Clear()
    uab.classes.SecureHeader:DumpAllHeaders(uabDumper)
    self:IterateActivators("Dump", uabDumper)
    uabDumper:Write()
end

function uabMain:PLAYER_DEAD()
	for _, activator in pairs(self.activators) do
	    activator:IAmDead()
	end
end

function uabMain:PLAYER_REGEN_ENABLED()
    --[[
    When we leave combat, we register and unregister any frames that requested to register/unregister
	during combat.
    --]]
    
	self:DelayedFrameRegister()
end

function uabMain:Enable()
    self:CreateClickActivators()
    self:DisableActivators()
    
    self.ccframesEnabled = true

    self:ConnectToUnitFrames()
	self:EnableActivators()

	-- Make sure that we show the correct layout and at the correct scale
    self:ScaleChanged()
    
    self:GetCfgDlg():GetActionsTab():ActivatorsChanged()
    uabLayoutManager:LayoutChanged(uab.db.profile.layout)
    uab.classes.SecureHeader:HideAllHeaders()
end

function uabMain:CreateClickActivators()
    self.numactivators = 0
    
	for key, val in pairs(uab.db.profile.activators) do
	    if key ~= "" then
            self:CreateActivator(key, val)
        else
            uab.db.profile.activators[key] = nil
        end
	end
    
    uab:RefreshActivatorList()
    
    self:GetCfgDlg():GetActionsTab():ActivatorsChanged()
end

function uabMain:GetCfgDlg()
    if not self.cfgdlg then
        self.cfgdlg = uab.classes.cfgdlg:new()
    end
    
    return self.cfgdlg
end

function uabMain:ActivatorExists(name)
    if self.activators[name] then
        return true
    end
end

function uabMain:AnyActivators(name)
    local any
    
    for name, activator in pairs(self.activators) do
        if name and activator then
            any = true
            break
        end
    end
    
    return any
end

function uabMain:NewActivator(name)
    local activation = self:FindNextAvailableActivation()

    if not activation then
        uab.error("No available activation click combinations available.")
        return
    end
        
    local activationdata = {}

	activationdata.enabled = true
    activationdata.button = activation.button
    activationdata.prefixshift = activation.prefixshift
    activationdata.prefixctrl = activation.prefixctrl
    activationdata.prefixalt = activation.prefixalt
    
    activationdata.groups = {}
    for _, info in pairs(uab.constants.groupInfo) do
        activationdata.groups[info.name] = {}
        activationdata.groups[info.name].enabled = true
    end
    
    uab.db.profile.activators[name] = activationdata

	self:CreateActivator(name, uab.db.profile.activators[name])
    
    local activator = self.activators[name]
    
    activator:Enable()

	--[[
	Hook this activator to every frame that is registered
	--]]
    for frame in pairs(self.ccframes) do
        if uabFrameGroup:GetFrameGroup(frame) then
            self:Debug("Registering frame '"..frame:GetName().."' in activator '"..name.."'")
            activator:RegisterFrame(frame)
        else
            self:Debug("Unable to find frame group for : "..frame:GetName())
        end
    end

    uab:RefreshActivatorList()
    
    self:GetCfgDlg():GetActionsTab():ActivatorsChanged()
end

function uabMain:GetActivator(name)
    return self.activators[name]
end

function uabMain:CopyActivatorTo(from, to)
    self.activators[from]:CopyToHostile(self.activators[to], "hostile")
    self.activators[from]:CopyTo(self.activators[to])
end

function uabMain:ClearActivator(name)
	self.activators[name]:Disable()
	self.activators[name] = nil
end

function uabMain:RemoveActivator(name)
    self:ClearActivator(name)

	uab.db.profile.activators[name] = nil
	
	uab:RefreshActivatorList()
    
    self:GetCfgDlg():GetActionsTab():ActivatorsChanged()
end

function uabMain:AddActivatorKey(name, key)
	local states = self:DisableActivators()
	
    uab.db.profile.activators[name]["prefix"..key] = true
    self.activators[name]:SetActivationData()

	self:EnableActivators(states)
end

function uabMain:RemoveActivatorKey(name, key)
	local states = self:DisableActivators()

    uab.db.profile.activators[name]["prefix"..key] = nil
    self.activators[name]:SetActivationData()

	self:EnableActivators(states)
end

function uabMain:ClearActivatorKeys(name)
	local states = self:DisableActivators()

    uab.db.profile.activators[name].prefixalt = nil
    uab.db.profile.activators[name].prefixctrl = nil
    uab.db.profile.activators[name].prefixshift = nil
    self.activators[name]:SetActivationData()

	self:EnableActivators(states)
end

function uabMain:SetActivatorButton(name, btn)
	local states = self:DisableActivators()

    uab.db.profile.activators[name].button = btn
    self.activators[name]:SetActivationData()

	self:EnableActivators(states)
end

function uabMain:DisableActivators()
	local states = {}
	self:IterateActivators(
		function(activator, states)
		    states[activator] = activator:Enabled()
		    activator:Disable()
		end,
		states)

	return states
end

function uabMain:EnableActivators(states)
	self:IterateActivators(
		function(activator, states)
			if not states or states[activator] then
				activator:Enable()
			end
		end,
		states)
end

function uabMain:DisableActivator(name)
    local activator = self.activators[name]
    
    if activator and activator:Enabled() then
        activator:Disable()
    end
end

function uabMain:EnableActivator(name)
    local activator = self.activators[name]
    
    if activator and not activator:Enabled() then
        activator:Enable()
    end
end

function uabMain:CreateActivator(name, data)
    self.numactivators = self.numactivators + 1
	self.activators[name] = uab.classes.ClickActivator:new(self.numactivators, name, data)

	uabFrameGroup:NewActivator(self.activators[name])
end

function uabMain:ConnectToUnitFrames()

    self:HookClickCastFrames()

	self:RegisterAllFrames()
end

function uabMain:RegisterAllFrames()
    -- Register all frames that snuck in before we did =)
    for frame in pairs(self.ccframes) do
		self:RegisterFrame(frame)
    end
end

function uabMain:UnregisterAllFrames()
    -- Unregister all the frames we currently have registered
    for frame in pairs(self.ccframes) do
		self:UnregisterFrame(frame)
    end
end

function uabMain:CCFNewIndex(t, k, v)
	if self.ccframesEnabled and not self.ccframesNewIndex then
	    if v == nil then
	        self:UnregisterFrame(k)
	    else
	        self:RegisterFrame(k)
	    end
	end
	
    --[[
	If there was someone loaded before us, and they catch
    ClickCastFrames changes, then let them have the event
    and let them do the rawset.  First, they need to process
    the event, and second, if they process it, they will
    either do the rawset or pass it to the next one in line
    as we are doing here.
    --]]
    
    if self.ccframesNewIndex then
        self.ccframesNewIndex(t,k,v)
    else
        if v == nil then
            rawset(self.ccframes, k, nil)
        else
            rawset(self.ccframes, k, v)
        end
    end
end

function uabMain:HookClickCastFrames()
	if not self.ccframesHooked then
	    self.ccframesHooked = true
	    
		ClickCastFrames = ClickCastFrames or {}
		self.ccframes = ClickCastFrames

	    local mt = getmetatable(ClickCastFrames)
	    if not mt then
	        mt = {}
	    end

	    self.ccframesNewIndex = mt.__newindex

        if self.ccframesNewIndex then
            --[[
            Another click-cast add-on is already loaded.  We
            should note this, and not attach ourselves.  Any
            further actions will be retained in case at some
            reload, the other click-cast add-on is not loaded
            but the actions will have no effect in the current
            session.
            --]]
            
            uab.print("Another click-casting add-on is already loaded.")
            uab.print("If you want to use UnitActionBars, please disable the other add-on.")
            uab.print("You may set up action sets, actions, and general settings, but those")
            uab.print("settings will not have any effect, as UnitActionBars will not attach")
            uab.print("to any frame to enable click-casting.")
        end
        
	    local newindex = function(t,k,v)
	        uab.main:CCFNewIndex(t, k, v)
	    end

	    mt.__newindex = newindex

	    ClickCastFrames = setmetatable(ClickCastFrames, mt)
	end
	
    local tbl = {
		PlayerFrame,
		PetFrame,
		PartyMemberFrame1,
		PartyMemberFrame2,
		PartyMemberFrame3,
		PartyMemberFrame4,
		PartyMemberFrame1PetFrame,
		PartyMemberFrame2PetFrame,
		PartyMemberFrame3PetFrame,
		PartyMemberFrame4PetFrame,
		TargetFrame,
		TargetofTargetFrame,
    }

    for i, frame in pairs(tbl) do
		if not ClickCastFrames[frame] then
            ClickCastFrames[frame] = true
        end
    end
end

function uabMain:Disable()
    self:DisconnectFromUnitFrames()
	self:DisableActivators()
    self:ClearActivators()
end

function uabMain:ClearActivators()
	for name, activator in pairs(self.activators) do
        self:ClearActivator(name)
	end
end

function uabMain:DisconnectFromUnitFrames()

	-- stop us from registering anyone while disconnecting
    self:UnhookClickCastFrames()

    -- Unregister all frames that are currently registered
    for frame in pairs(self.ccframes) do
		self:UnregisterFrame(frame)
    end
end

function uabMain:UnhookClickCastFrames()
	--[[
	We don't actually disconnect from the metatable, as someone may
	have connected after us.  Instead, we just indicate that we should
	not process any registrations now.  They will still be in the table
	when we are re-enabled, and will be registered at that time.
	--]]

	self.ccframesEnabled = nil
end

function uabMain:RegisterFrame(frame)
    if InCombatLockdown() then
        self:DelayRegisterFrame(frame)
        return
    end

    if uabFrameGroup:GetFrameGroup(frame) then
        self:EnableClickCastFrame(frame)
    else
        self:Debug("Unable to find frame group for : "..frame:GetName())
    end
end

function uabMain:EnableClickCastFrame(frame)
    self:Debug("Registering frame : "..frame:GetName())
    
	self:IterateActivators("RegisterFrame", frame)
end

function uabMain:UnregisterFrame(frame)
    if InCombatLockdown() then
        self:DelayUnregisterFrame(frame)
        return
    end

    self:DisableClickCastFrame(frame)
end

function uabMain:DisableClickCastFrame(frame)
	self:IterateActivators("UnregisterFrame", frame)
end

function uabMain:DelayRegisterFrame(frame)
	self.combatRegistrations[frame] = true
end

function uabMain:DelayUnregisterFrame(frame)
	self.combatUnregistrations[frame] = true
end

function uabMain:DelayedFrameRegister()
    self:RegisterDelayedFrames()
    self:UnregisterDelayedFrames()
end

function uabMain:RegisterDelayedFrames()
	for frame, _ in pairs(self.combatRegistrations) do
        uab.print("UAB: Registering queued frame "..frame:GetName())
		self:RegisterFrame(frame)
        self.combatRegistrations[frame] = nil
	end
end

function uabMain:UnregisterDelayedFrames()
	for frame, _ in pairs(self.combatUnregistrations) do
        uab.print("UAB: Unregistering queued frame "..frame:GetName())
		self:UnregisterFrame(frame)
        self.combatUnregistrations[frame] = nil
	end
end

function uabMain:ActionsChanged(activator)
    uabLayoutManager:LayoutSettingsChanged()
end

function uabMain:SecureHeaderChanged(groupname)
	self:IterateActivators("SecureHeaderChanged", groupname)
end

function uabMain:ScaleChanged(groupname)
	self:IterateActivators("ScaleChanged", groupname)
end

function uabMain:PositionChanged(groupname)
	self:IterateActivators("PositionChanged", groupname)
end

function uabMain:DirectCastChanged()
	self:IterateActivators("DirectCastChanged")
end

function uabMain:StayOpenChanged()
	self:IterateActivators("StayOpenChanged")
end

function uabMain:FastChangeUnitChanged()
    self.secureheader:FastChangeUnitChanged()
	self:IterateActivators("FastChangeUnitChanged")
end

function uabMain:FrameGroupsChanged()
	--[[
	Because we may have disabled UAB on some frame groups, and
    enabled it on others, we should unregister and reregister
    all the frames.
	--]]
    self:UnregisterAllFrames()
	self:RegisterAllFrames()
end

function uabMain:HighlightBorderChanged()
	self:IterateActivators("HighlightBorderChanged")
end

function uabMain:FindNextAvailableActivation()
    --[[
    Find the first click combination that is not already used by
    some action set.  Search for the combination in the following order:

    The modifiers are tried in the following order for each button:
        
    shift
    ctrl
    alt
    shift-ctrl
    shift-alt
    ctrl-alt
    ctrl-shift-alt
    
    The buttons are tried in the following order:
    
    RightButton
    LeftButton
    MiddleButton
    Button4
    Button5
    
    If none of these match, the button alone is tried in the following
    order:
    
    MiddleButton
    Button4
    Button5
    RightButton
    LeftButton

    If all of these are taken, an error is generated, and no action set
    can be created.    
    --]]
    
    for idx, entry in pairs(uab.classes.ActionsTab.activations) do
        if not self:AnyActivatorWithActivation(entry) then
            return entry
        end
    end
end
            
function uabMain:AnyActivatorWithActivation(entry)
    for name, activator in pairs(self.activators) do
        local settings = uab.db.profile.activators[name]
        
        if settings.button == entry.button and
            settings.prefixshift == entry.prefixshift and
            settings.prefixctrl == entry.prefixctrl and
            settings.prefixalt == entry.prefixalt then
            return true
        end
    end
end

function uabMain:IterateActivators(fcn, context1, context2, context3)
	for _, activator in pairs(self.activators) do
		local result

		if type(fcn) == "string" then
		    result = activator[fcn](activator, context1, context2, context3)
		else
			result = fcn(activator, context1, context2, context3)
		end

		if result then
		    return result
		end
	end
end

