--[[
A UnitGroupHeader maintains a set of ActionButton and ActionButtonNone
instances.  It controls the state transitions for those buttons.  A
UnitGroupHeader also maintains a set of UnitGroupControl instances.  In WoW
2.01, there is a single UnitGroupControl for each UnitGroupHeader.  In WoW
2.02, there will be a UnitGroupControl for each unit in the associated
UnitGroup.

The UnitGroupHeader defines the valid states, and button-unit associations for
each of the ActionButton instances it maintains.  Each ActionButton is told
the state-unit associations, so that when the button is clicked, the action
associated with the button applies to the correct unit based on the current
state and those associations.

The UnitGroupHeader is associated with a single UnitGroup
The UnitGroupHeader maintains a list of ActionButton instances
The UnitGroupHeader maintains a single ActionButtonNone instance
The UnitGroupHeader maintains a single ActionSetLayout instance
--]]
uab.classes.UnitGroupHeader = uab.mixins.aceOO.Class()
uab.classes.UnitGroupHeader.nextState = 1

function uab.classes.UnitGroupHeader.NextState()
    local nextState = uab.classes.UnitGroupHeader.nextState
    uab.classes.UnitGroupHeader.nextState = uab.classes.UnitGroupHeader.nextState + 1

    return nextState
end

function uab.classes.UnitGroupHeader.prototype:init(group, numunits, actions)
	uab.classes.UnitGroupHeader.super.prototype.init(self)

	self.group = group
    self.numcontrols = numunits * uab.constants.controlsperunit
	self.controls = {}
	self.nextstate = 0

	--[[
	Create the header frame.  We create one header from (the only one for the group), and
	multiple controls, and the loop counter is passed to the control to differentiate
    between control instances.
	--]]
    
    self.name = self.group.qualifiedname
    self.states = {
        offset = self.group.activator.statemultiplier*uab.constants.maxactivatorcontrols,
        min = uab.constants.groupInfo[self.group.name].minstate,
        max = uab.constants.groupInfo[self.group.name].maxstate
    }
    
    uab:Debug("Header: "..self.name.." : staterange = "..self.states.offset + self.states.min.."-"..self.states.offset + self.states.max)

	self.cancelbtn = uabActionButton:NewCancel(self)
	self.actionbtns = {}

    if actions then
    	for idx = 1, 24, 1 do
    	    if actions[idx] then
    	        self:SetAction(idx, actions[idx])
    		end
    	end
    end

    self:HostileActionsChanged()    
	self:SecureHeaderChanged()
    self:PositionChanged()
    self:ScaleChanged()
end

function uab.classes.UnitGroupHeader.prototype:Dump(dumper)
    dumper:Dump("Header : states = ("..self.states.offset..", "..self.states.min..", "..self.states.max..")")
    dumper:Dump("Controls")
    dumper:IncIndent()
    self:IterateControls("Dump", dumper)
    dumper:DecIndent()
    dumper:Dump("Buttons")
    dumper:IncIndent()
    self.cancelbtn:Dump(dumper)
    for idx = 1, 24, 1 do
        local btn = self.actionbtns[idx]
        if btn and btn.isenabled then
            btn:Dump(dumper)
        end
    end
    dumper:DecIndent()
end

function uab.classes.UnitGroupHeader.prototype:NewControl()
	local state = self.nextstate
	if state <= self.states.max then
		self.nextstate = self.nextstate + 1
        return uab.classes.UnitGroupControl:new(self, self.states.offset + self.states.min + state, self.group.name)
	end
end

function uab.classes.UnitGroupHeader.prototype:HostileActionsChanged()
    for idx = 1, 24, 1 do
        self:SetHostileAction(idx, self.group.activator:GetHostileAction(idx))
    end
end

function uab.classes.UnitGroupHeader.prototype:SecureHeaderChanged()
	local currentheader = self.secureheader

    -- This group is independent from other groups.	
    if uab.db.profile.independent[self.group.name] then
        --[[
        If the activator is also independent, then this group should be
        completely unique.  Otherwise, use the group-specific header.
        --]]
        if uab.db.profile.activators[self.group.activator.name].independent then
            self.secureheader = uab.classes.SecureHeader:GetHeader(self.group.name, self.group.activator.name)
        else
            self.secureheader = uab.classes.SecureHeader:GetHeader(self.group.name)
        end
    else
    	self.secureheader = self.group.activator:GetSecureHeader()
	end

    self.frame = self.secureheader.frame

	if self.secureheader ~= currentheader then
		--[[
		Reparent all of this guys action buttons, and no action button, to
		the new header, if it is different.
		--]]
		self:SetButtonParent()
        
        --[[
        Reposition the buttons relative to the new header.
        --]]
        self:LayoutChanged()
		    
	    --[[
	    Make sure that all the state-anchor and scale are set properly on the
		header frame, since we changed headers.  Also, make sure that
        each control references the correct header for that control, and that
        the correct fast-change unit settings are in place on each control as
        well.
	    --]]
        self:IterateControls("SecureHeaderChanged")
        
        self:FastChangeUnitChanged()
        self:PositionChanged()
        self:ScaleChanged()
	end
end

function uab.classes.UnitGroupHeader.prototype:SetButtonParent()
	if self.cancelbtn then
        self.cancelbtn:SetParent()
    end

	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:SetParent()
	end	
end

function uab.classes.UnitGroupHeader.prototype:IAmDead()
    --[[
    Set the header frame state attribute to 0 to (hopefully) cause our cancel
	and action buttons to hide (because of the state change to a hidden state).
	Since we are dead, we are no longer in combat, so this *should* be ok to
	do.
    --]]
    self.secureheader:IAmDead()
    self.cancelbtn:Hide()
    for _, btn in pairs(self.actionbtns) do
        btn:Hide()
    end
end

function uab.classes.UnitGroupHeader.prototype:SetAction(number, action)
    if action then
        if not self.actionbtns[number] then
	        self.actionbtns[number] = uabActionButton:New(self, action, number)
        else
	        self.actionbtns[number]:SetAction(action)
        end
    elseif self.actionbtns[number] then
        self.actionbtns[number]:SetAction(action)
    end
end

function uab.classes.UnitGroupHeader.prototype:SetHostileAction(number, action)
    if action then
        if not self.actionbtns[number] then
	        self.actionbtns[number] = uabActionButton:New(self, nil, number)
        end
    	self.actionbtns[number]:SetHostileAction(action)
    elseif self.actionbtns[number] then
        self.actionbtns[number]:SetHostileAction(action)
    end
end

function uab.classes.UnitGroupHeader.prototype:LayoutChanged()
    local layouts = {}
    local grouplayoutname = self.group:GetLayoutName()
    local grouplayout = self.group:GetLayout()
    local cnt = { count = 0 }
    
    self:IterateControls(
		function(control, layouts)
            local fgmodule
            local fgmodulename
			local fgname
            local fginfo
            local layoutname

            if control.unitframe then
                fgmodule = uabFrameGroup:GetFrameGroup(control.unitframe)
                if fgmodule then
                    fgmodulename = fgmodule.name
    			    fgname, fginfo = fgmodule:OptionsForFrame(control.unitframe)
                    layoutname = fginfo.layout
                end
                
                cnt.count = cnt.count + 1
            end
                        
            --[[
            If the frame group does not define its own layout, then use the
            layout defined by the group (which may be the global layout).
            --]]
            if not layoutname then
                layoutname = grouplayoutname
                layout = grouplayout
            else
                local layoutmgr = uabLayoutManager:Get(layoutname)
                
                --[[
                If the layout defined by the frame group is not one that is
                currently loaded, then use the layout defined by the group
                (which may be the global layout).
                --]]
                if not layoutmgr then
                    layoutname = grouplayoutname
                    layout = grouplayout
                else
                    layout = layoutmgr:GetLayout("fgroup:"..fgmodulename..":"..fgname)
                    --[[
                    If the layout is missing from the layout manager, then
                    use the layout defined by the group (which may be the global
                    layout).
                    --]]
                    if not layout then
                        layoutname = grouplayoutname
                        layout = grouplayout
                    end
                end
            end

            if not layouts[layout] then
                layouts[layout] = {}
            end

            --[[
            Add this control to the set of controls to use the layout we
            just identified.
            --]]            
            table.insert(layouts[layout], control)
		end,
        layouts)

    if cnt.count > 0 then
    	self.cancelbtn:PrepareForLayouts()
    
    	for _, actionbtn in pairs(self.actionbtns) do
    		actionbtn:PrepareForLayouts()
    	end	
        
        for layout, controls in pairs(layouts) do
            local states
                    
            for idx, control in pairs(controls) do
                if states then
                    states = states .. ","
                else
                    states = ""
                end
                
                states = states .. control.state
            end
        
            layout:ArrangeButtons(self.cancelbtn, self.actionbtns, states)
        end
    
    	self.cancelbtn:LayoutsDone()
    
    	for _, actionbtn in pairs(self.actionbtns) do
    		actionbtn:LayoutsDone()
    	end
    end
end

function uab.classes.UnitGroupHeader.prototype:IsRegisteredFrame(frame)
    return self:IterateControls("IsRegisteredFrame", frame)
end

function uab.classes.UnitGroupHeader.prototype:UnregisterFrame(frame)
    return self:IterateControls(
		function(control, frame)
			if control:IsRegisteredFrame(frame) then
				control:UnassignFrame()
				return true
			end
		end,
		frame)
end

function uab.classes.UnitGroupHeader.prototype:CanRegisterFrame(frame)
    local unit = SecureButton_GetModifiedUnit(frame)
    local canregister
    local groupname = self.group.name
    
	if not unit and groupname == "partyraid" then
		canregister = true
	elseif unit then
        canregister = (groupname == unit)
        if not canregister then
            if string.sub(unit, 1, 5) == "party" then
            	canregister = (groupname == "partyraid")
            elseif string.sub(unit, 1, 4) == "raid" then
            	canregister = (groupname == "partyraid")
            end
        end
	end

	return canregister and self:HaveAvailableControl(frame)
end

function uab.classes.UnitGroupHeader.prototype:RegisterFrame(frame)
    local control = self:NextControl(frame)
    local cnt = { count = 0 }

    if not control then return end

	self.controls[control] = true
	
    control:AssignFrame(frame)
    self:LayoutChanged()
end

function uab.classes.UnitGroupHeader.prototype:ActivateControls()
    self:IterateControls("Enable")
end

function uab.classes.UnitGroupHeader.prototype:DeactivateControls()
    self:IterateControls("Disable")
end

function uab.classes.UnitGroupHeader.prototype:SwitchToSingleAction(action, control)
    if control then
        control:SwitchToDirectAction(action)
    else
        self:IterateControls("SwitchToDirectAction", action)
    end
end

function uab.classes.UnitGroupHeader.prototype:SwitchToButtons(control)
    if control then
        control:SwitchToClickAction()
    else
        self:IterateControls("SwitchToClickAction")
    end
end

function uab.classes.UnitGroupHeader.prototype:SwitchToSingleHostileAction(action, control)
    if control then
        control:SwitchToDirectHostileAction(action)
    else
        self:IterateControls("SwitchToDirectHostileAction", action)
    end
end

function uab.classes.UnitGroupHeader.prototype:SwitchToHostileButtons(control)
    if control then
        control:SwitchToHostileClickAction()
    else
        self:IterateControls("SwitchToHostileClickAction")
    end
end

function uab.classes.UnitGroupHeader.prototype:TimerEvent(elapsed)
	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:TimerEvent(elapsed)
	end	
end

function uab.classes.UnitGroupHeader.prototype:ScaleChanged()
    self:IterateControls("ScaleChanged")

    if self.group:UsingDefaultScale() then
        self.secureheader:SetStateScaleRange(self.states.offset + self.states.min, self.states.offset + self.states.max)
    else
        self.secureheader:SetStateScaleRange(self.states.offset + self.states.min, self.states.offset + self.states.max, self.group:GetScale())
    end
end

function uab.classes.UnitGroupHeader.prototype:PositionChanged()
    self:IterateControls("PositionChanged")

    local pos = self.group:GetPosition()
    if self.group:UsingDefaultPosition() then
        self.secureheader:SetStatePositionRange(self.states.offset + self.states.min, self.states.offset + self.states.max)
    else
        self.secureheader:SetStatePositionRange(self.states.offset + self.states.min, self.states.offset + self.states.max, pos.x, pos.y, pos.relpoint, pos.point)
    end
end

function uab.classes.UnitGroupHeader.prototype:StayOpenChanged()
	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:StayOpenChanged()
	end	
end

function uab.classes.UnitGroupHeader.prototype:FastChangeUnitChanged()
    self:IterateControls("FastChangeUnitChanged")
end

function uab.classes.UnitGroupHeader.prototype:HighlightBorderChanged()
	for _, actionbtn in pairs(self.actionbtns) do
		actionbtn:HighlightBorderChanged()
	end	
end

--[[---------------------------------------------------------------------------
Helper methods, to be used only by the class itself
-----------------------------------------------------------------------------]]

function uab.classes.UnitGroupHeader.prototype:ClearScaleStateRange()
    self.secureheader:SetStateScaleRange(self.states.offset + self.states.min, self.states.offset + self.states.max)
end

function uab.classes.UnitGroupHeader.prototype:ClearPositionStateRange()
    self.secureheader:SetStatePositionRange(self.states.offset + self.states.min, self.states.offset + self.states.max)
end

function uab.classes.UnitGroupHeader.prototype:HaveAvailableControl(frame)
	local unused_control = self:IterateControls(
		function(control)
			if not control.unitframe then
				return true
			end
		end)

	return unused_control or uabFrameGroup:HaveFrameControl(frame, self.group.activator)
end

function uab.classes.UnitGroupHeader.prototype:NextControl(frame)
    local unused_control = self:IterateControls(
		function(control)
			if not control.unitframe then
				return control
			end
		end)

	return unused_control or uabFrameGroup:NextFrameControl(frame, self.group.activator)
end

function uab.classes.UnitGroupHeader.prototype:IterateControls(fcn, context1, context2, context3)
	for control, _ in pairs(self.controls) do
		local result

		if type(fcn) == "string" then
		    result = control[fcn](control, context1, context2, context3)
		else
			result = fcn(control, context1, context2, context3)
		end
		
		if result then
		    return result
		end
	end
end
