--[[
This class controls the arrangement of the ActionButtonNone and ActionButton
instances for a UnitGroupHeader.

There are one or more IActionSetLayout instances allocated for each
UnitGroupHeader.  How many depends on whether the layout is changed while
in-game.  The only layouts allocated are ones that were selected in-game.  Each
UnitGroupHeader keeps track of the active ActionSetLayout to use.
--]]
uabBarLayoutManager.Layout = uab.mixins.aceOO.Class(IActionSetLayout)

function uabBarLayoutManager.Layout.prototype:init(context, settings)
    uabBarLayoutManager.Layout.super.prototype.init(self)

    self.config = uabBarLayoutManager.LayoutConfig:new(self, settings)
    self.context = context
    self.name = "uab.Bar_"..context
end

function uabBarLayoutManager.Layout.prototype:LayoutSettingsChanged()
    uabLayoutManager:LayoutSettingsChanged()
end

function uabBarLayoutManager.Layout.prototype:ArrangeButtons(cancelbtn, actionbtns, states)
    local settings = self.config.settings
    local found
    local havecancel

    uabLayoutManager:Debug(self.name .. " : ArrangeButtons")

    --[[
    First, reset all positioning information
    --]]
    self:FormatCancelBtn(cancelbtn)

    for idx = 1, 24, 1 do
        if actionbtns[idx] then
            local btn = actionbtns[idx].frame
            if not self.buttonsize then
                self.buttonsize = { x = btn:GetWidth(), y = btn:GetHeight() }
            end
            self:FormatActionBtn(actionbtns[idx])
            if not found then
                found = actionbtns[idx].isenabled
            end
        end
    end
	
    --[[
    Now, position the cancel button centered relative to the header (the
    parent of the cancel button)
    --]]
    if not self.config.settings.hidecancel and found then
        cancelbtn:AddShowStates(states)
        cancelbtn:SetPointForStates(states, "CENTER", 0, 0)
        havecancel = true
    end

    local overflowModifier = { x = 0, y = 0 }
    local buttonRows = 24 / settings.growLimit

    if buttonRows * settings.growLimit < 24 then
        buttonRows = buttonRows + 1
    end

    if settings.overflowClockwise == "true" then
        if settings.growDirection == uab.constants.grow.up then
            overflowModifier.x = 1
            overflowModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.down then
            overflowModifier.x = -1
            overflowModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.right then
            overflowModifier.x = 0
            overflowModifier.y = -1
        else
            overflowModifier.x = 0
            overflowModifier.y = 1
        end
    else
        if settings.growDirection == uab.constants.grow.up then
            overflowModifier.x = -1
            overflowModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.down then
            overflowModifier.x = 1
            overflowModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.right then
            overflowModifier.x = 0
            overflowModifier.y = 1
        else
            overflowModifier.x = 0
            overflowModifier.y = -1
        end
    end

    local growModifier = { x = 0, y = 0 }
    local start = { x = 0, y = 0 }
    local spacing = { x = 0, y = 0 }
    
    if settings.growDirection == uab.constants.grow.up or settings.growDirection == uab.constants.grow.down then
        spacing.x = settings.overflowSpacing
        spacing.y = settings.growSpacing
    else
        spacing.x = settings.growSpacing
        spacing.y = settings.overflowSpacing
    end

    if settings.placement == uab.constants.placement.top then
        if settings.growDirection == uab.constants.grow.up then
            growModifier.x = 0
            growModifier.y = 1
        elseif settings.growDirection == uab.constants.grow.left then
            growModifier.x = -1
            growModifier.y = 0
        else    -- uab.constants.grow.right
            growModifier.x = 1
            growModifier.y = 0
        end

        start.x = 0
        start.y = 1
    elseif settings.placement == uab.constants.placement.right then
        if settings.growDirection == uab.constants.grow.right then
            growModifier.x = 1
            growModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.up then
            growModifier.x = 0
            growModifier.y = 1
        else    -- uab.constants.grow.down
            growModifier.x = 0
            growModifier.y = -1
        end

        start.x = 1
        start.y = 0
    elseif settings.placement == uab.constants.placement.bottom then
        if settings.growDirection == uab.constants.grow.down then
            growModifier.x = 0
            growModifier.y = -1
        elseif settings.growDirection == uab.constants.grow.right then
            growModifier.x = 1
            growModifier.y = 0
        else    -- uab.constants.grow.left
            growModifier.x = -1
            growModifier.y = 0
        end

        start.x = 0
        start.y = -1
    elseif settings.placement == uab.constants.placement.left then
        if settings.growDirection == uab.constants.grow.left then
            growModifier.x = -1
            growModifier.y = 0
        elseif settings.growDirection == uab.constants.grow.up then
            growModifier.x = 0
            growModifier.y = 1
        else    -- uab.constants.grow.down
            growModifier.x = 0
            growModifier.y = -1
        end

        start.x = -1
        start.y = 0
    end

    if not havecancel then
        start.x = 0
        start.y = 0
    end
    
    local button = actionbtns[1]
    local positions = {}

    positions[1] = { x = start.x * (uab.constants.buttonWidth + spacing.x), y = start.y * (uab.constants.buttonHeight + spacing.y) }

    if button and button.isenabled then
        button:AddShowStates(states)
        button:SetPointForStates(states, "CENTER", positions[1].x, positions[1].y)
    end

    for pos = 2, settings.growLimit, 1 do
        positions[pos] = { x = 0, y = 0 }

        positions[pos].x = (start.x + (growModifier.x * (pos - 1))) * (uab.constants.buttonWidth + spacing.x)
        positions[pos].y = (start.y + (growModifier.y * (pos - 1))) * (uab.constants.buttonHeight + spacing.y)

        button = actionbtns[pos]

        if button and button.isenabled then
            button:AddShowStates(states)
            button:SetPointForStates(states, "CENTER", positions[pos].x, positions[pos].y)
        end
    end

    for row = 2, buttonRows, 1 do
        self:ArrangeRow(states, positions, row, overflowModifier, spacing, cancelbtn, actionbtns)
    end
end

function uabBarLayoutManager.Layout.prototype:ArrangeRow(states, positions, row, placementMod, spacing, cancelbtn, actionbtns)
	local settings = self.config.settings
    local ofs = (row - 1) * settings.growLimit
    local relofs = ofs - settings.growLimit

    for pos = 1, settings.growLimit, 1 do
        local btnpos = pos + ofs
        local btn = actionbtns[btnpos]
        local relbtnpos = pos + relofs
        local relpos = positions[relbtnpos]

        positions[btnpos] = { x = 0, y = 0 }

        positions[btnpos].x = relpos.x + (placementMod.x * (uab.constants.buttonWidth + spacing.x))
        positions[btnpos].y = relpos.y + (placementMod.y * (uab.constants.buttonHeight + spacing.y))

        if btn and btn.isenabled then
            btn:AddShowStates(states)
            btn:SetPointForStates(states, "CENTER", positions[btnpos].x, positions[btnpos].y)
        end
    end
end

function uabBarLayoutManager.Layout.prototype:GetConfig()
	return self.config
end

function uabBarLayoutManager.Layout.prototype:SetSettings(settings)
    self.config:SettingsChanged(settings)
end

function uabBarLayoutManager.Layout.prototype:FormatCancelBtn(cancelbtn)
    -- Nothing to do here at this time.  Would be a good idea to resize
    -- the cancel button frame, hide the text frame for the button frame
end

function uabBarLayoutManager.Layout.prototype:FormatActionBtn(btn)
    -- Nothing to do here at this time.  Would be a good idea to resize
    -- the cancel button frame, hide the text frame for the button frame
end

